@@ -1,10 +1,10 @@
 use crate::{
-    body::{self, Bytes, Full, HttpBody},
+    body::{Bytes, HttpBody},
     extract::{rejection::*, FromRequest, RequestParts},
-    response::{IntoResponse, Response},
     BoxError,
 };
 use async_trait::async_trait;
+use axum_core::response::{IntoResponse, Response};
 use http::{
     header::{self, HeaderValue},
     StatusCode,
@@ -15,9 +15,16 @@ use std::ops::{Deref, DerefMut};
 /// JSON Extractor / Response.
 ///
 /// When used as an extractor, it can deserialize request bodies into some type that
-/// implements [`serde::Deserialize`]. If the request body cannot be parsed, or it does not contain
-/// the `Content-Type: application/json` header, it will reject the request and return a
-/// `400 Bad Request` response.
+/// implements [`serde::Deserialize`]. The request will be rejected (and a [`JsonRejection`] will
+/// be returned) if:
+///
+/// - The request doesn't have a `Content-Type: application/json` (or similar) header.
+/// - The body doesn't contain syntactically valid JSON.
+/// - The body contains syntactically valid JSON but it couldn't be deserialized into the target
+/// type.
+/// - Buffering the request body fails.
+///
+/// See [`JsonRejection`] for more details.
 ///
 /// # Extractor example
 ///
@@ -96,10 +103,30 @@ where
     type Rejection = JsonRejection;
 
     async fn from_request(req: &mut RequestParts<B>) -> Result<Self, Self::Rejection> {
-        if json_content_type(req)? {
+        if json_content_type(req) {
             let bytes = Bytes::from_request(req).await?;
 
-            let value = serde_json::from_slice(&bytes).map_err(InvalidJsonBody::from_err)?;
+            let value = match serde_json::from_slice(&bytes) {
+                Ok(value) => value,
+                Err(err) => {
+                    let rejection = match err.classify() {
+                        serde_json::error::Category::Data => JsonDataError::from_err(err).into(),
+                        serde_json::error::Category::Syntax | serde_json::error::Category::Eof => {
+                            JsonSyntaxError::from_err(err).into()
+                        }
+                        serde_json::error::Category::Io => {
+                            if cfg!(debug_assertions) {
+                                // we don't use `serde_json::from_reader` and instead always buffer
+                                // bodies first, so we shouldn't encounter any IO errors
+                                unreachable!()
+                            } else {
+                                JsonSyntaxError::from_err(err).into()
+                            }
+                        }
+                    };
+                    return Err(rejection);
+                }
+            };
 
             Ok(Json(value))
         } else {
@@ -108,33 +135,29 @@ where
     }
 }
 
-fn json_content_type<B>(req: &RequestParts<B>) -> Result<bool, HeadersAlreadyExtracted> {
-    let content_type = if let Some(content_type) = req
-        .headers()
-        .ok_or_else(HeadersAlreadyExtracted::default)?
-        .get(header::CONTENT_TYPE)
-    {
+fn json_content_type<B>(req: &RequestParts<B>) -> bool {
+    let content_type = if let Some(content_type) = req.headers().get(header::CONTENT_TYPE) {
         content_type
     } else {
-        return Ok(false);
+        return false;
     };
 
     let content_type = if let Ok(content_type) = content_type.to_str() {
         content_type
     } else {
-        return Ok(false);
+        return false;
     };
 
     let mime = if let Ok(mime) = content_type.parse::<mime::Mime>() {
         mime
     } else {
-        return Ok(false);
+        return false;
     };
 
     let is_json_content_type = mime.type_() == "application"
         && (mime.subtype() == "json" || mime.suffix().map_or(false, |name| name == "json"));
 
-    Ok(is_json_content_type)
+    is_json_content_type
 }
 
 impl<T> Deref for Json<T> {
@@ -162,26 +185,25 @@ where
     T: Serialize,
 {
     fn into_response(self) -> Response {
-        let bytes = match serde_json::to_vec(&self.0) {
-            Ok(res) => res,
-            Err(err) => {
-                return Response::builder()
-                    .status(StatusCode::INTERNAL_SERVER_ERROR)
-                    .header(
-                        header::CONTENT_TYPE,
-                        HeaderValue::from_static(mime::TEXT_PLAIN_UTF_8.as_ref()),
-                    )
-                    .body(body::boxed(Full::from(err.to_string())))
-                    .unwrap();
-            }
-        };
-
-        let mut res = Response::new(body::boxed(Full::from(bytes)));
-        res.headers_mut().insert(
-            header::CONTENT_TYPE,
-            HeaderValue::from_static(mime::APPLICATION_JSON.as_ref()),
-        );
-        res
+        match serde_json::to_vec(&self.0) {
+            Ok(bytes) => (
+                [(
+                    header::CONTENT_TYPE,
+                    HeaderValue::from_static(mime::APPLICATION_JSON.as_ref()),
+                )],
+                bytes,
+            )
+                .into_response(),
+            Err(err) => (
+                StatusCode::INTERNAL_SERVER_ERROR,
+                [(
+                    header::CONTENT_TYPE,
+                    HeaderValue::from_static(mime::TEXT_PLAIN_UTF_8.as_ref()),
+                )],
+                err.to_string(),
+            )
+                .into_response(),
+        }
     }
 }
 
@@ -249,4 +271,19 @@ mod tests {
         assert!(valid_json_content_type("application/cloudevents+json").await);
         assert!(!valid_json_content_type("text/json").await);
     }
+
+    #[tokio::test]
+    async fn invalid_json_syntax() {
+        let app = Router::new().route("/", post(|_: Json<serde_json::Value>| async {}));
+
+        let client = TestClient::new(app);
+        let res = client
+            .post("/")
+            .body("{")
+            .header("content-type", "application/json")
+            .send()
+            .await;
+
+        assert_eq!(res.status(), StatusCode::BAD_REQUEST);
+    }
 }
