@@ -262,7 +262,7 @@
 //! # };
 //! ```
 //!
-//! The downside to this approach is that its a little more verbose than using
+//! The downside to this approach is that it's a little more verbose than using
 //! extensions.
 //!
 //! # Building integrations for axum
@@ -310,12 +310,16 @@
 //! `http1` | Enables hyper's `http1` feature | Yes
 //! `http2` | Enables hyper's `http2` feature | No
 //! `json` | Enables the [`Json`] type and some similar convenience functionality | Yes
+//! `matched-path` | Enables capturing of every request's router path and the [`MatchedPath`] extractor | Yes
 //! `multipart` | Enables parsing `multipart/form-data` requests with [`Multipart`] | No
+//! `original-uri` | Enables capturing of every request's original URI and the [`OriginalUri`] extractor | Yes
 //! `tower-log` | Enables `tower`'s `log` feature | Yes
 //! `ws` | Enables WebSockets support via [`extract::ws`] | No
 //!
 //! [`TypedHeader`]: crate::extract::TypedHeader
+//! [`MatchedPath`]: crate::extract::MatchedPath
 //! [`Multipart`]: crate::extract::Multipart
+//! [`OriginalUri`]: crate::extract::OriginalUri
 //! [`tower`]: https://crates.io/crates/tower
 //! [`tower-http`]: https://crates.io/crates/tower-http
 //! [`tokio`]: http://crates.io/crates/tokio
@@ -327,7 +331,6 @@
 //! [examples]: https://github.com/tokio-rs/axum/tree/main/examples
 //! [`Router::merge`]: crate::routing::Router::merge
 //! [`axum::Server`]: hyper::server::Server
-//! [`OriginalUri`]: crate::extract::OriginalUri
 //! [`Service`]: tower::Service
 //! [`Service::poll_ready`]: tower::Service::poll_ready
 //! [`Service`'s]: tower::Service
@@ -389,9 +392,11 @@
 #[macro_use]
 pub(crate) mod macros;
 
-mod add_extension;
+mod extension;
 #[cfg(feature = "json")]
 mod json;
+#[cfg(feature = "headers")]
+mod typed_header;
 mod util;
 
 pub mod body;
@@ -405,10 +410,6 @@ pub mod routing;
 #[cfg(test)]
 mod test_helpers;
 
-pub use add_extension::AddExtension;
-#[allow(deprecated)]
-pub use add_extension::AddExtensionLayer;
-
 #[doc(no_inline)]
 pub use async_trait::async_trait;
 #[cfg(feature = "headers")]
@@ -419,11 +420,17 @@ pub use http;
 #[doc(no_inline)]
 pub use hyper::Server;
 
+#[doc(inline)]
+pub use self::extension::Extension;
 #[doc(inline)]
 #[cfg(feature = "json")]
 pub use self::json::Json;
 #[doc(inline)]
 pub use self::routing::Router;
 
+#[doc(inline)]
+#[cfg(feature = "headers")]
+pub use self::typed_header::TypedHeader;
+
 #[doc(inline)]
 pub use axum_core::{BoxError, Error};
