@@ -1,6 +1,5 @@
-use super::{IntoResponse, Response};
-use crate::body::{boxed, Empty};
-use http::{header::LOCATION, HeaderValue, StatusCode, Uri};
+use axum_core::response::{IntoResponse, Response};
+use http::{header::LOCATION, HeaderValue, StatusCode};
 use std::convert::TryFrom;
 
 /// Response that redirects the request to another location.
@@ -15,7 +14,7 @@ use std::convert::TryFrom;
 /// };
 ///
 /// let app = Router::new()
-///     .route("/old", get(|| async { Redirect::permanent("/new".parse().unwrap()) }))
+///     .route("/old", get(|| async { Redirect::permanent("/new") }))
 ///     .route("/new", get(|| async { "Hello!" }));
 /// # async {
 /// # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
@@ -41,7 +40,7 @@ impl Redirect {
     /// If `uri` isn't a valid [`HeaderValue`].
     ///
     /// [mdn]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/303
-    pub fn to(uri: Uri) -> Self {
+    pub fn to(uri: &str) -> Self {
         Self::with_status_code(StatusCode::SEE_OTHER, uri)
     }
 
@@ -55,7 +54,7 @@ impl Redirect {
     /// If `uri` isn't a valid [`HeaderValue`].
     ///
     /// [mdn]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/307
-    pub fn temporary(uri: Uri) -> Self {
+    pub fn temporary(uri: &str) -> Self {
         Self::with_status_code(StatusCode::TEMPORARY_REDIRECT, uri)
     }
 
@@ -66,36 +65,15 @@ impl Redirect {
     /// If `uri` isn't a valid [`HeaderValue`].
     ///
     /// [mdn]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/308
-    pub fn permanent(uri: Uri) -> Self {
+    pub fn permanent(uri: &str) -> Self {
         Self::with_status_code(StatusCode::PERMANENT_REDIRECT, uri)
     }
 
-    /// Create a new [`Redirect`] that uses a [`302 Found`][mdn] status code.
-    ///
-    /// This is the same as [`Redirect::temporary`] ([`307 Temporary Redirect`][mdn307]) except
-    /// this status code is older and thus supported by some legacy clients that don't understand
-    /// the newer one. Many clients wrongly apply [`Redirect::to`] ([`303 See Other`][mdn303])
-    /// semantics for this status code, so it should be avoided where possible.
-    ///
-    /// # Panics
-    ///
-    /// If `uri` isn't a valid [`HeaderValue`].
-    ///
-    /// [mdn]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/302
-    /// [mdn307]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/307
-    /// [mdn303]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/303
-    #[deprecated(
-        note = "This results in different behavior between clients, so Redirect::temporary or Redirect::to should be used instead"
-    )]
-    pub fn found(uri: Uri) -> Self {
-        Self::with_status_code(StatusCode::FOUND, uri)
-    }
-
     // This is intentionally not public since other kinds of redirects might not
     // use the `Location` header, namely `304 Not Modified`.
     //
     // We're open to adding more constructors upon request, if they make sense :)
-    fn with_status_code(status_code: StatusCode, uri: Uri) -> Self {
+    fn with_status_code(status_code: StatusCode, uri: &str) -> Self {
         assert!(
             status_code.is_redirection(),
             "not a redirection status code"
@@ -103,17 +81,13 @@ impl Redirect {
 
         Self {
             status_code,
-            location: HeaderValue::try_from(uri.to_string())
-                .expect("URI isn't a valid header value"),
+            location: HeaderValue::try_from(uri).expect("URI isn't a valid header value"),
         }
     }
 }
 
 impl IntoResponse for Redirect {
     fn into_response(self) -> Response {
-        let mut res = Response::new(boxed(Empty::new()));
-        *res.status_mut() = self.status_code;
-        res.headers_mut().insert(LOCATION, self.location);
-        res
+        (self.status_code, [(LOCATION, self.location)]).into_response()
     }
 }
